/* BASES DE DATOS 2 */
/* TECNOLOGO EN INFORMATICA */
/* PRIMER OBLIGATORIO */

/* TABLAS Y PRIMARY KEYS */
	
CREATE TABLE actores (
    id_actor integer NOT NULL AUTO_INCREMENT,
    nombre character varying(45) NOT NULL,
    apellido character varying(45) NOT NULL,
	CONSTRAINT actores_pkey PRIMARY KEY (id_actor)
);

CREATE TABLE actores_de_peliculas (
    id_actor smallint NOT NULL,
    id_pelicula smallint NOT NULL,
	CONSTRAINT actor_de_pelicula_pkey PRIMARY KEY (id_actor, id_pelicula)
);

CREATE TABLE alquileres (
    id_pelicula smallint NOT NULL,
    id_sucursal smallint NOT NULL,
    id_cliente smallint NOT NULL,
    fecha timestamp NOT NULL,
    id_personal smallint NOT NULL,
    fecha_devolucion timestamp null,
	CONSTRAINT alquileres_pkey PRIMARY KEY (id_pelicula, id_cliente, id_sucursal, id_personal, fecha)
);

CREATE TABLE categorias (
    id_categoria integer NOT NULL AUTO_INCREMENT,
    categoria character varying(25) NOT NULL,
	CONSTRAINT categorias_pkey PRIMARY KEY (id_categoria)
);

CREATE TABLE categorias_de_peliculas (
    id_pelicula smallint NOT NULL,
    id_categoria smallint NOT NULL,
	CONSTRAINT categoria_de_pelicula_pkey PRIMARY KEY (id_pelicula, id_categoria)
);

CREATE TABLE ciudades (
    id_ciudad integer NOT NULL AUTO_INCREMENT,
    ciudad character varying(50) NOT NULL,
    id_pais smallint NOT NULL,
	CONSTRAINT ciudades_pkey PRIMARY KEY (id_ciudad)
);

CREATE TABLE clientes (
    id_cliente integer NOT NULL AUTO_INCREMENT,
    id_sucursal smallint NOT NULL,
    nombre character varying(45) NOT NULL,
    apellido character varying(45) NOT NULL,
    email character varying(50),
    direccion character varying(50) NOT NULL,
    distrito character varying(20) NOT NULL,
    id_ciudad smallint NOT NULL,
    codigo_postal character varying(10),
    telefono character varying(20) NOT NULL,
    activo boolean DEFAULT true NOT NULL,
	CONSTRAINT clientes_pkey PRIMARY KEY (id_cliente)
);

CREATE TABLE idiomas (
    id_idioma integer NOT NULL AUTO_INCREMENT,
    idioma character(20) NOT NULL,
	CONSTRAINT idiomas_pkey PRIMARY KEY (id_idioma)
);

CREATE TABLE idiomas_de_peliculas (
    id_idioma smallint NOT NULL,
    id_pelicula smallint NOT NULL,
	CONSTRAINT idioma_de_pelicula_pkey PRIMARY KEY (id_idioma, id_pelicula)
);

CREATE TABLE inventario (
    id_pelicula smallint NOT NULL,
    id_sucursal smallint NOT NULL,
    cant_ejemplares smallint NOT NULL,
	CONSTRAINT inventario_pkey PRIMARY KEY (id_pelicula, id_sucursal)
);

CREATE TABLE pagos (
    id_pelicula_alquilo smallint NOT NULL,
    id_cliente_alquilo smallint NOT NULL,
    id_sucursal_alquilo smallint NOT NULL,
    id_personal_alquilo smallint NOT NULL,
    fecha_alquilo timestamp NOT NULL,
    id_personal_recibe_pago smallint NOT NULL,
    monto numeric(5,2) NOT NULL,
    fecha timestamp NOT NULL,
	CONSTRAINT pagos_pkey PRIMARY KEY (id_pelicula_alquilo, id_cliente_alquilo, id_sucursal_alquilo, id_personal_alquilo, fecha_alquilo, fecha)
);

CREATE TABLE paises (
    id_pais integer NOT NULL AUTO_INCREMENT,
    pais character varying(50) NOT NULL,
	CONSTRAINT paises_pkey PRIMARY KEY (id_pais)
);

CREATE TABLE peliculas (
    id_pelicula integer NOT NULL AUTO_INCREMENT,
    titulo character varying(255) NOT NULL,
    descripcion text,
    anio year(4),
    id_idioma_original smallint,
    duracion_alquiler smallint DEFAULT 3 NOT NULL,
    costo_alquiler numeric(4,2) DEFAULT 99.8 NOT NULL,
    duracion smallint,
    costo_reemplazo numeric(5,2) DEFAULT 399.8 NOT NULL,
    clasificacion ENUM ('G', 'PG', 'PG-13','R', 'NC-17') DEFAULT 'G',
    contenidos_extra text,
	CONSTRAINT peliculas_pkey PRIMARY KEY (id_pelicula)
);

CREATE TABLE personal (
    id_personal integer NOT NULL AUTO_INCREMENT,
    nombre character varying(45) NOT NULL,
    apellido character varying(45) NOT NULL,
    direccion character varying(50) NOT NULL,
    distrito character varying(20) NOT NULL,
    id_ciudad smallint NOT NULL,
    codigo_postal character varying(10),
    telefono character varying(20) NOT NULL,
    email character varying(50),
    id_sucursal smallint NOT NULL,
    activo boolean DEFAULT true NOT NULL,
    usuario character varying(16) NOT NULL,
    password character varying(40),
	CONSTRAINT personal_pkey PRIMARY KEY (id_personal)
);

CREATE TABLE sucursales (
    id_sucursal integer NOT NULL AUTO_INCREMENT,
    id_encargado smallint NOT NULL,
    direccion character varying(50) NOT NULL,
    distrito character varying(20) NOT NULL,
    id_ciudad smallint NOT NULL,
    codigo_postal character varying(10),
    telefono character varying(20) NOT NULL,
	CONSTRAINT sucursal_pkey PRIMARY KEY (id_sucursal)
);
 
/* FOREIGN KEYS */

ALTER TABLE actores_de_peliculas
    ADD CONSTRAINT actor_de_pelicula_actor_id_fkey FOREIGN KEY (id_actor) REFERENCES actores(id_actor) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE actores_de_peliculas
    ADD CONSTRAINT actor_de_pelicula_pelicula_id_fkey FOREIGN KEY (id_pelicula) REFERENCES peliculas(id_pelicula) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE pagos
    ADD CONSTRAINT alquiler_fkey FOREIGN KEY (id_pelicula_alquilo, id_cliente_alquilo, id_sucursal_alquilo, id_personal_alquilo, fecha_alquilo) REFERENCES alquileres(id_pelicula, id_cliente, id_sucursal, id_personal, fecha) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE alquileres
    ADD CONSTRAINT alquileres_alquiler_fkey FOREIGN KEY (id_pelicula, id_sucursal) REFERENCES inventario(id_pelicula, id_sucursal) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE alquileres
    ADD CONSTRAINT alquileres_id_cliente_fkey FOREIGN KEY (id_cliente) REFERENCES clientes(id_cliente) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE alquileres
    ADD CONSTRAINT alquileres_id_personal_fkey FOREIGN KEY (id_personal) REFERENCES personal(id_personal) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE categorias_de_peliculas
    ADD CONSTRAINT categoria_de_pelicula_categoria_id_fkey FOREIGN KEY (id_categoria) REFERENCES categorias(id_categoria) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE categorias_de_peliculas
    ADD CONSTRAINT categoria_de_pelicula_pelicula_id_fkey FOREIGN KEY (id_pelicula) REFERENCES peliculas(id_pelicula) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ciudades
    ADD CONSTRAINT ciudad_id_pais_fkey FOREIGN KEY (id_pais) REFERENCES paises(id_pais) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE clientes
    ADD CONSTRAINT cliente_ciudad_fkey FOREIGN KEY (id_ciudad) REFERENCES ciudades(id_ciudad) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE clientes
    ADD CONSTRAINT cliente_sucursal_fkey FOREIGN KEY (id_sucursal) REFERENCES sucursales(id_sucursal) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE idiomas_de_peliculas
    ADD CONSTRAINT idioma_de_pelicula_idioma_id_fkey FOREIGN KEY (id_idioma) REFERENCES idiomas(id_idioma) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE idiomas_de_peliculas
    ADD CONSTRAINT idioma_de_pelicula_pelicula_id_fkey FOREIGN KEY (id_pelicula) REFERENCES peliculas(id_pelicula) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE inventario
    ADD CONSTRAINT inventario_id_pelicula_fkey FOREIGN KEY (id_pelicula) REFERENCES peliculas(id_pelicula) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE inventario
    ADD CONSTRAINT inventario_id_sucursal_fkey FOREIGN KEY (id_sucursal) REFERENCES sucursales(id_sucursal) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE pagos
    ADD CONSTRAINT pagos_id_personal_recibe_pago_fkey FOREIGN KEY (id_personal_recibe_pago) REFERENCES personal(id_personal) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE peliculas
    ADD CONSTRAINT pelicula_original_id_idioma_fkey FOREIGN KEY (id_idioma_original) REFERENCES idiomas(id_idioma) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE personal
    ADD CONSTRAINT personal_ciudad_fkey FOREIGN KEY (id_ciudad) REFERENCES ciudades(id_ciudad) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE personal
    ADD CONSTRAINT personal_sucursal_fkey FOREIGN KEY (id_sucursal) REFERENCES sucursales(id_sucursal) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE sucursales
    ADD CONSTRAINT sucursal_id_diudad_fkey FOREIGN KEY (id_ciudad) REFERENCES ciudades(id_ciudad) ON UPDATE CASCADE ON DELETE RESTRICT;
